<?PHP

/**
* Main block class for sitenavigation block
*
* @package sitenavigation
* @category blocks
* @author Rosario Carco, Fachhochschule Nordwestschweiz
* @version 1.9.10, 2012112101
* @uses $CFG->dirroot . '/blocks/sitenavigation/HTML_TreeMenu-sitenavigation/Tree.php'
* @uses $CFG->dirroot . '/blocks/sitenavigation/HTML_TreeMenu-mycourses/TreeMenu.php'
*
* I took the myCourses Block which was originally released as part of the FN
* Moodle.
*
* This block was partially revised and enhanced by Julian Ridden and Mario
* Schellenberger
*
* Nevertheless I decided to reuse the original FN-Code which was labeled as
* follows, the file's date being 2006-oct-04, but integrating the mentioned
* enhancements and revisions:
*
* $Id: block_FN_my_courses.php,v 1.1.2.1 2006/06/23 14:59:40 mchurch Exp $
*
* See the following threads: http://moodle.org/mod/forum/discuss.php?d=67494 and
*                            http://moodle.org/mod/forum/discuss.php?d=103703
*
* I modified the code to address the following issues:
*
* Development history:
*
* Date      | Author | Text (newest on top, please!)
************+********+******************************
*
* 2011/11/26 xx
*
* 2012/11/16 Cr - TODO: the mycmyc button (blue) acted as a main switch for teachers to show/hide all courses
*                       but it could be better to use active (orange) inactive (grey) buttons instead to show/hide these two and
*                       use the mycmyc only for admins from now on.
* 2012/11/20 Cr - backporting green button to toggle on/off old courses, from mycourses 23.01:
* 2012/11/14 Cr - added notvisitedsince to allow students and teachers to
*                 toggle on/off the display of old courses, not visited since
*                 the given number of days. According admin oprion added. Idea:
*                 implement user GUI that allows students and teachers
*                 to show only ACTIVE courses, i.e. courses they actually attend
*                 or ALL courses. As teachers often forget to unenroll students
*                 or must leave them enrolled because of the grades, students can
*                 toggle the display now. It is up to the admin to set how many
*                 days are used to decide whether courses are actually attended
*                 or havent been accessed. E.g. at our university courses are
*                 attended during 6 months which is 180 days. So courses not
*                 attended in the last 180 days could be considered as old/not
*                 actually attended courses. Unfortunately when starting the new
*                 semester this would still show the courses of the previous
*                 one. Narrowing this to 30 or fewer days would mean that courses
*                 of the current semester the students did not visit during those
*                 30 days would not show up either. So they would have to toggle
*                 on ALL courses first to get the link back and to access the
*                 course so that it is displayed again. So it is up to the Admin
*                 to choose and set a good compromise in that option.
*                 I think another approach could be to set the starting and the
*                 end date of a semester/term and then display all the courses
*                 as actually attended courses which have been visited at least
*                 once inbetween those dates. Let me know if you would prefer
*                 rather this solution exclusively or as a second option to
*                 select.
*
* 2012/01/18 Cr - changed all variable and classnames to all lowercase as
*                 required by the Moodle programming guidelines and the
*                 new Modules & Plugins Database checks.
* 2011/12/06 Cr - a user asked for another admin option to show either the
*                 siteNavigation tree OR the myCoures tree OR both.
* 2011/12/05 Cr - a user asked for another admin option, to show only active
*                 i.e.  not hidden courses. He wants to use only the siteNavigation
*                 without combining it with the myCourses Block where hidden
*                 courses would not be shown.
* 2011/06/15 Cr - Ok, I just found a workaround for all those who want to
*                 preserve the blue, orange and gray button functionality
*                 also in the popUp window. But it is at the cost of
*                 performance because I have to rewrite the file on the
*                 fly and you would have to click on the link
*                 "show myCourses in popUp window" again, and as the
*                 browser is lazy, you have to right-click in the popUp
*                 window contents to force the content to RELOAD again.
*
* 2011/06/06 Cr - NOTE: actually siteNavigation and myCourses contents is
*                       generated and written always to the same files in
*                       $CFG->dataroot/1/sess_xyz_siteNavigation.html and
*                       $CFG->dataroot/1/sess_xyz_myCourses.html.
*                       To avoid any delays when users register themselves in
*                       new courses, the myCourses-List must still be generated
*                       on the fly (unless you would accept a gap of 5 to 30
*                       minutes until new courses appeared in the myCourses
*                       list). The siteNavigation list could be regarded
*                       as something that does not change often during a day.
*                       This file could really be generated once or twice a day
*                       for all users to use.
*                       For the moment being, I decided to still generate both
*                       contents and files when the users click on the according
*                       links. But you have to set up cron-jobs that purge the
*                       files in $CFG->dataroot/1/ with something like
*                       rm yourMoodleDataroot/1/sess_*.html
*                       WE STILL gain in performance as the floating popUp
*                       windows will hold the navigation lists and therefore
*                       have not to be generated on every course page the
*                       users visit.
*                       And on the other hand, if siteNavigation should become
*                       a bottleneck on a large scale site, we could introduce
*                       an admin option to delete and generate the file every
*                       x minutes a day.
*               - NOTE: If users register in new courses they have to click on
*                       the show myCourses in a popUp window link to trigger
*                       the creation of the new list.
*               - NOTE: The $CFG->dataroot/session directory is purged by a
*                       Moodle cron job but unfortunately I could not access
*                       it with Moodle's file.php script which works only on
*                       existing course-id directories starting with 1 upwards.
*               - NOTE: For those who can not set up own cron jobs on hosted
*                       Moodle servers, I left the option to display both trees
*                       in one Block, as before. No files are written and hence
*                       performance remains at the stage of the 1.9.07 line.
*               - NOTE: The buttons blue, orange and gray to toggle on/off
*                       myCourses, active and inactive courses are only
*                       available in block-mode. I am not sure there is a
*                       solution to use them in the floatin popUp windows.
*                       If there IS a solution, I will implement it later on.
*
* 2011/06/06 Cr - NOTE: I did not succeed in embedding the next code snippets
*                       to control the window's/tab's focus. The HTML
*                       unfortunately breaks, I guess due to the underlying
*                       HTML-Tree function calls. I would not want to adapt
*                       those functions anyway because we are going to use
*                       YUI-Menu functions in the next release.
*
* 2011/05/05 Cr - Found this solution to force the windows to get the focus
*                 after loading the Moodle Course page, but this still does
*                 not work for TABs. So I added this code in TreeMenu.js
*                 and Tree.js just for those users who use windows instead of
*                 tabs:
*
*<Script Language="JavaScript">
*
*function open_link_in_window(myurl, myname) {
*   var new_window = window.open(myurl, myname);
*   new_window.focus();
*}
*
*</Script>
*
*</head><body>
*
*
*<A HREF="https://moodle.fhnw.ch/moodleTest/course/view.php?id=4" onClick="open_link_in_window('https://moodle.fhnw.ch/moodleTest/course/view.php?id=4', 'myCourses');return false">Click Here!</A>
*<A HREF="https://moodle.fhnw.ch/moodleTest/course/view.php?id=5" onClick="open_link_in_window('https://moodle.fhnw.ch/moodleTest/course/view.php?id=5', 'myCourses');return false">Click Here!</A>
*
*<A HREF="https://moodle.fhnw.ch/moodleTest/course/view.php?id=839" onClick="open_link_in_window('https://moodle.fhnw.ch/moodleTest/course/view.php?id=839', 'myCourses');return false">Click Here!</A>
*<A HREF="https://moodle.fhnw.ch/moodleTest/course/view.php?id=840" onClick="open_link_in_window('https://moodle.fhnw.ch/moodleTest/course/view.php?id=840', 'myCourses');return false">Click Here!</A>
*
*</body></html>
*
* Further code snippet to write into a form field:
* echo "<script type=\"text/javascript\">window.opener.document.edit.textfield.value='".$someValue."';</script>";
*
* 2011/05/04 Cr - Displaying the trees in a separate window has a major
*                 caveat: when you click on a link, where should it be opened?
*                 In the same window would obviously remove the tree and you
*                 would have to navigate back. Opening in a new window/tab
*                 seems to be the best bet. But then another question arises:
*                 should every link be openend in the same window/tab or
*                 should every link open a separate browser window/tab??
*                 I decided to open every page IN THE SAME window/tab. But
*                 this handy solution has another nasty behaviour: when the
*                 window is opened for the first time it comes up on the front
*                 and has the focus for the user to continue with his work.
*                 But if the window gets into the background and the user
*                 clicks on a link in the myCourses or siteNavigation tree,
*                 the link is opened in the window in the background without
*                 beeing brought on top. This can be a frustrating user
*                 experience if the user does not know he should either close
*                 the window/tab or bring it to the front himself. In TABs
*                 focus does not come either, but at least you see the tab
*                 flashing and loading a new page.
* 2011/03/17 Cr - This is another intermediate version because a new problem
*                 arised: with more than 4'000 courses nested deeply in more
*                 than 1'000 categories, the myCourses-Block simply was too
*                 small to offer a decent and comfortable navigation. So we
*                 decided to show the siteNavigation AND the myCourses blocks
*                 in different popUp windows, keeping only a minimal block
*                 with roughly the search box for courses, the link to
*                 Moodle's "all courses" list, the link to popup the
*                 siteNavigation and another one to popup the myCourses list.
*                 Still using the old code, we will not gain much in speed on
*                 medium to large scale sites. BUT THERE ARE some
*                 enhancements: we store the myCourses and the siteNavigation
*                 trees into files now, so that they can be used on every
*                 course page again because they are composed only once per
*                 user session. The siteNavigation file is created only if
*                 it does not exist yet. So you can decide on how many times
*                 a day the siteNavigation file should be created anew by
*                 having a purging cron-job which deletes the file, lets say
*                 every 2 hours, or even every 15 Minutes, if you want it to
*                 reflect also latest changes in the whole category and course
*                 structure. [this option is actually postponed to the next
*                 version]
*                 This way we can save a lot of resources compared
*                 with the previous versions which rendered everything on every
*                 course page on the fly with more than 3'000 DB queries if
*                 both blocks were present on any page.
*
*               - NOTE: this is already a merged version of both blocks into
*                 one. But it is not the new sitenav which will also use the
*                 YUI-libraries to render the hierarchical navigation. And
*                 navigation could be enhanced if we do not use nested folders
*                 indented on every new horizontal line, but keeping only ONE
*                 indent and closing all other folders like this:
*
*                 CAT 1 (closed)
*                 CAT 2 (open)
*                   CAT 2a (closed)
*                   CAT 2b (closed)
*                 CAT 3 (closed)
*
*                 as soon as you click CAT 2a you get:
*
*                 CAT 1(closed)
*                 CAT 2(open)
*                 CAT 2a (open)
*                   CAT 2a1 (closed)
*                   CAT 2a2 (closed)
*                 CAT 3 (closed)
*
*                 So that only ONE category/folder is expanded at a time which
*                 avoids the horizontal shift to the right the deeper we dive.
*
*               - NOTE: for conveniency reasons I simply copied the two
*                 versions of HTML-TreeMenu Libraries so as to continue to
*                 avoid naming conflicts and allow separate cookie entries
*                 to save the expanded/collapsed status of the folders.
*
* 2010/07/20 Cr - This is the final version before we start over with a merged
*                 version of both the blocks myCourses and siteNavigation.
*                 You will find any further development and comments in the
*                 siteNavigation thread on moodle.org Forums.
*
* 2010/07/15 Cr - Tried also htmlentities($string, ENT_QUOTES) instead of
*                 htmlspecialchars() but this breaks the tool tip texts also.
*                 So for the moment I leave it with htmlspecialchars.
*
*               - This is the code sent to the client browser including some
*                 other special chars. The apostrophe/single quote in the
*                 title will break:
*
*newNode_8_3_2_1 = newNode_8_3_2.addItem(new TreeNode1(' Bases de l\'apostrophe  optomtrie',
*                    'folder.gif', 'https://moodle2.test.fhnw.ch/course/category.php?id=441
*                    title="Bases de l'apostrophe  optomtrie"',
*                    false, true, 'treeMenuDefault', '', 'folder-expanded.gif'));
*
*               - This is the code sent to the client browser if we use
*                 htmlspecialchars(), where the apostrophe gets encoded to &#039;
*
*newNode_8_3_2_1 = newNode_8_3_2.addItem(new TreeNode1(' Bases de l\'apostrophe  optomtrie',
*                    'folder.gif', 'https://moodle2.test.fhnw.ch/course/category.php?id=441"
*                    title="Bases de l&#039;apostrophe  optomtrie',
*                    false, true, 'treeMenuDefault', '', 'folder-expanded.gif'));
*
*               - Anyway it seems we have
*                 still to avoid some special chars in category or course
*                 names. See my proposed workaraounds below.
*
* 2009/11/16 Cr - started reengineering because the queries in loops and
*                 recursive calls yield too many queries on larger sites
*                 with more than 1000 courses. HAS BEEN POSTPONED TO VERSION
*                 AFTER merging the two blocks into one.
*
* 2009/11/16 Cr - trying with htmlspecialchars($string, ENT_QUOTES) to convert
*                 also single and double quotes. This works, but I saw that
*                 Tree.php does already a str_replace("'", "\\'", $nodeObj->text)
*                 to avoid such problems, so I use it only on the tooltip
*                 texts which break if they contain an apostrophe/single quote
*                 character and maybe also with other special chars. If you
*                 have such names see my proposed workarounds hereafter.
*
* 2009/11/13 Cr - investigated the code to avoid problems with
*                 course names containing special chars like the APOSTROPHE!
*                 If you
*                 use one of those characters in your category or course names
*                 the title="some tool tip text" inside the <a href="url"
*                 title="some tool tip text">link to course or category</a> will
*                 break and not display anything in the block.
*
*                 So either you use one of those tricks:
*                 - you remove the special char which causes the trouble from
*                   your category or course name or you replace it with another
*                   substitute like the Back-Tick for Apostrophe like l`avion
*                   instead of l'avion
*                 - you encode the special char in your category or course
*                   name with &apos; or &uuml;
*                 - you insert a str_replace() to replace the critical char
*                   only for the tool tip texts
*                 - you comment the lines in this code which provide the tool
*                   tip help to avoid displaying it. I might add another
*                   config option for this later on. The lines to comment are:
*
*                   title="'.format_string($course->shortname); or
*                   title="'.format_string($course->fullname); or
*                   title="'.format_string($SITE->shortname); or
*                   title="'.format_string($SITE->fullname); or
*                   title="'.format_string($catnode->name);
*
* 2009/11/11 Cr - added format_string() to correctly show multilanguage cat
*                 and course names
*
* 2009/10/19 Cr - replaced $siteNavByCategories with
*                 $CFG->block_siteNavigation_sitenavigationbycategory
*
* 2009/05/26 Cr - Started to implement get_my_remote_courses() to add mNet
*                 courses to the list of courses. As I can not test this code
*                 on my moodle, this code remains EXPERIMENTAL until
*                 thoroughly tested.
*               - new option to toggle short or long names in the display
*                 line or tool tip help
*               - new option to show/hide active links for folders to
*                 navigate to corresponding Moodle category page
*               - new option to show a flat list of courses instead of a
*                 hierarchical tree-structure with folders
*               - @todo convert some or all of these options to individual
*                       user options. As this block is a user block, I feel
*                       that the user should be able to configure it to his
*                       taste. On the other hand, if this block is combined
*                       with the siteNavigation block, it still makes sense
*                       that the site-Admin chooses the best combination of
*                       options for everybody.
*               - new option to show/hide a course search box
*               - checked all indentations by 4 spaces and split comment
*                 lines to not exceed 80 chars horizontally
*               - changed all string- and other variables to lowercase
*
* 2008/09/11 Cr - crossported config setting to show/hide
*                 scroll bars
* 2008/09/04 Cr - option added to show/hide "all courses" link in the footer
* 2008/09/03 Cr - do not display "all courses" link if loginRequired is set
*               - bug fix: even with loginRequired navigation was visible
* 2008/08/19 Cr - minor bug fixes and spanish language added
* 2008/08/18 Cr - I started to modify the new code base of myCourses to get a
*                 siteNavigation block. In this release there is still no interface
*                 to configure the three parameters in the MANAGE BLOCKS Admin section.
*                 For the moment I use local variables to simulate and configure these
*                 Block Settings. See below on what these params are.
* 2008/08/08 Cr - I have been using this myCourses Block now for a couple of months and
*                 also in the moodle-thread, where I uploaded Version 1.9.02, 2008052601,
*                 noone complained. So it seems to be stable and I removed all old FN Code,
*                 which previously was only commented out. I will put this version as
*                 first stable release in the thread http://moodle.org/mod/forum/discuss.php?d=67494
*                 and then start working on the new block called siteNavigation.
*
* Starting Works and Design:
*
* 2008/05/09 Cr - last tests with code of Julian Ridden
* 2008/05/13 Cr - decided to use original FN code. Adapted code to run fine in 1.9
* 2008/05/14 Cr - adapted to reflect Moodle 1.9 programming of blocks according to
*                 http://docs.moodle.org/en/Development:Blocks
*                 - removed all code relicts of FN and all code not strictly necessary
*                 - removed also all errors displayed with DEBUGGING on
*                 - added language support for GE, FR, IT (maybe spanish and portuguese, still open)
*
* @todo some points of the next section are still open
*
*                 - made TWO different blocks, so that they can be used concurrently in
*                   different courses and the Front Page:
*
*                 a) the original block name and purpose remane the same: myCourses
*                     - show only courses of logged in user
*                       - @todo What should be displayed if user is the site-administrator?
*                         - actually admins can choose to show/hide active and inactive courses,
*                           which makes the most sense imho, no myCourses in which they could be
*                           enrolled. Maybe the original code's intent was another one, which
*                           I could not deduce/reconstruct from the code. Let me know if there
*                           is something I missed or if I should restore the previous logic.
*                         - actually teachers can choose to show/hide myCourses, i.e. only
*                           courses in which they teach. Part of those courses may be set
*                           to "not available to students", i.e. those courses are not visible =
*                           hidden = inactive. Teachers are not allowed to see all courses,
*                           so in the block's GUI I only show two option buttons: show/hide myCourses
*                           and show/hide inactive courses.
*                         - actually normal users can see only the courses they are enrolled in.
*                           There is no GUI to allow them to set according options.
*
*
*                 b) changed name of second block to better reflect its purpose to: siteNavigation
*                    this block offers three params/settings:
*
*                    - siteNavByCategories -> show only categories, no courses for a fast display
*                      of the whole category-tree (loading 500 courses and more takes 7 seconds
*                      and more)
*
*                    - siteNavByCourses -> show all categories AND all courses (only appropriate
*                      for small sites with few courses, maybe less than 100)
*                      - @todo I am looking at the code of the built in block_admin_tree which
*                        also uses open/closed folders in a hierarchical tree. I guess that this
*                        code is faster to render the whole tree than the HTML_TreeMenu-1.2.0
*                        actually used.
*
*                    - loginRequested -> if YES, only logged in users may use it
*                                     -> if NO, everyone may use it to navigate, login prompt
*                                        appears only when entering a course.
*
*
*/

require_once($CFG->dirroot.'/blocks/sitenavigation/HTML_TreeMenu-sitenavigation/Tree.php');//Cr: changed file and class names
                                                                                           //to avoid conflicts with myCourses block

require_once($CFG->dirroot.'/blocks/sitenavigation/HTML_TreeMenu-mycourses/TreeMenu.php');//Cr: original for myCourses

class block_sitenavigation extends block_base {

    /**
     * Standard block API function for initializing block instance
     * @return void
     */

function init() {
    $this->title = get_string('blocktitle','block_sitenavigation');//Cr: must be
//initialized here and can be dynamically changed in the specialization
//function, see MoodleDocs:Development:Blocks

    $this->version = 2012112001;
}

/**
 * Standard block API function for initializing/preparing/changing block settings/variables
 * See block_course_summary.php for a good example
 *
 * @uses $COURSE
 * @uses $SITE
 * @return void
 */

function specialization() {
//At this point, $this->instance and $this->config are available
//for use. We can now change the title, etc. to whatever we want. E.g.
//$this->title = $this->config->variable_holding_the_title

    global $CFG, $COURSE, $SITE;//Cr: $course replaced with $COURSE

//Cr: check and initialize needed block config option variables for
//    sitenavigation

   if (!isset ($CFG->block_sitenavigation_sitenavigationbycategory) ) {
     $CFG->block_sitenavigation_sitenavigationbycategory = true;//Cr: show only
   }                                                            //   categories

   if (!isset ($CFG->block_sitenavigation_loginrequired)) {
     $CFG->block_sitenavigation_loginrequired = true;//Cr: login required
   }

   if (!isset ($CFG->block_sitenavigation_showallcourseslink)) {
     $CFG->block_sitenavigation_showallcourseslink = false;//Cr: no display
   }

    if (!isset ($CFG->block_sitenavigation_withscrollbars)) {
        $CFG->block_sitenavigation_withscrollbars = true;//Cr: show scrollbars
    }

    if (!isset ($CFG->block_sitenavigation_showlongnames)) {
        $CFG->block_sitenavigation_showlongnames = false;//Cr: display only
//short names and long names in the tool tip help
    }

    if (!isset ($CFG->block_sitenavigation_showcategorylinks)) {
        $CFG->block_sitenavigation_showcategorylinks = true;//Cr: display links
    }

    if (!isset ($CFG->block_sitenavigation_showflatlist)) {
        $CFG->block_sitenavigation_showflatlist = false;//Cr: show tree/folders
    }

    if (!isset ($CFG->block_sitenavigation_showsearchbox)) {
        $CFG->block_sitenavigation_showsearchbox = false;//Cr: no search box
    }

    if (!isset ($CFG->block_sitenavigation_show)) {
        $CFG->block_sitenavigation_show = true;//Cr: show sitenavigation
    }

    if (!isset ($CFG->block_sitenavigation_showinpopupwindow)) {
        $CFG->block_sitenavigation_showsearchbox = false;//Cr: do not display in popUpWindow
    }

    if (!isset ($CFG->block_sitenavigation_showhiddencourses)) {
        $CFG->block_sitenavigation_showhiddencourses = true;//Cr: show hidden courses
    }


//Cr: the same for mycourses block

    if (!isset ($CFG->block_sitenavigation_mycourses_withscrollbars)) {
        $CFG->block_sitenavigation_mycourses_withscrollbars = true; //Cr: show scrollbars
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_showallcourseslink)) {
        $CFG->block_sitenavigation_mycourses_showallcourseslink = false;//Cr: don't display it
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_showlongnames)) {
        $CFG->block_sitenavigation_mycourses_showlongnames = false;//Cr: display only short
//names and long names in the tool tip help
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_showcategorylinks)) {
        $CFG->block_sitenavigation_mycourses_showcategorylinks = true;//Cr: display links
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_showflatlist)) {
        $CFG->block_sitenavigation_mycourses_showflatlist = true;//Cr: display no tree/folders
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_showsearchbox)) {
        $CFG->block_sitenavigation_mycourses_showsearchbox = false;//Cr: display no search box
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_adminshowmycourses)) {
        $CFG->block_sitenavigation_mycourses_adminshowmycourses = false;//Cr: do not display
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_adminshowactivecourses)) {
        $CFG->block_sitenavigation_mycourses_adminshowactivecourses = false;//Cr: do not display
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_adminshowinactivecourses)) {
        $CFG->block_sitenavigation_mycourses_adminshowinactivecourses = false;//Cr: do not display
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_show)) {
        $CFG->block_sitenavigation_mycourses_show = true;//Cr: show mycourses
    }

    if (!isset ($CFG->block_sitenavigation_mycourses_showinpopupwindow)) {
        $CFG->block_sitenavigation_mycourses_showinpopupwindow = false;//Cr: do not display in popUpWindow
    }
    
    if (!isset ($CFG->block_sitenavigation_mycourses_notvisitedsince)) {
        $CFG->block_sitenavigation_mycourses_notvisitedsince = 0;//Cr: do not display courses that have not been accessed during the last x days
                                                                 //Note: 0 is default and means to show ALL courses in which the users are enrolled
    }
    

    if ($this->instance->pageid == SITEID) {//Cr: if this block appears on the
//front page which is course id # 1 and category # 0 in the course table,
        $this->course = $SITE;//start with $SITE which is initialized by
                              //get_site() in lib/setup.php
                              //which normally returns 1
    } else {

//Cr: else block appears on a course page, so start with this course
        $this->course = $COURSE;//Cr: $course replaced with $COURSE

//Cr: According to MoodleDocs:Development:Blocks you could also use
//$this->course = get_record('course', 'id', $this->instance->pageid);
    }

//Cr: initialize local variables
    $this->isadmin = isadmin();//Cr: to avoid further calls
    $this->isteacher = false;

    $this->mycmyc = false;//Cr: these values get initialized again below
    $this->mycact = false;//    with set_admin_options and set_teacher_options
    $this->mycinact = false;//  respectively
    $this->mycold = false;

//Cr: the next line tests whether the logged in user is also teacher in any course
//From the cross-ref: Determines if a user is a teacher in any course, or an
//                    admin
//bool isteacherinanycourse ([int $userid = 0], [bool $includeadmin = true])
//Set this to 0 if you would just like to test against the currently logged in
//user

    if (isteacherinanycourse()) {//Cr: teachers see ONLY courses they teach in
                                 //    and may choose to show/hide them in the
                                 //    header of the block
        $this->isteacher = true;//Cr: changed name of member variable
                                //    $this->marker to $this->isteacher to better
                                //    reflect admin and teacher options
    }

    $this->set_user_options();//Cr: as students and teachers may show/hide courses they do not attend
                              //    we call only this unified function now
}

/**
 * Standard block API function to return in which formats this block may be used
 *
 * @return array formats
 */

function applicable_formats() {
// The block can be used in all course types, but not in modules/activities
    return array(
        'all' => true,
        'site' => true,//Cr: this is probably already included in 'all'
        'mod' => false);
}

/**
 * Standard block API function to return the block's preferred width
 *
 * @return integer width
 */

function preferred_width() {//Cr: has to be coordinated with Mario's pix width
                            //    setting in TreeMenu.php and Tree.php
    return 210;//The default is 180, the admin_tree on the front page is 210
}

/**
 * Standard block API function for telling if block has configuration
 * @return boolean
 */

function has_config() {//Cr: settings.php is ready to use
    return true;
}

/**
 * Standard block API function for printing/composing/returning block content
 *
 * @uses $CFG
 * @var string $enrol to retain "all courses" text
 * @return object $this->content
 */

function get_content() {
    global $CFG;//$USER, $CFG, $THEME, $SESSION;//Cr: use as needed

//tjhunt:
// This prevents your block from recalculating its content more than once before the page
// is displayed to the user. Unless you KNOW that there is a VERY SPECIFIC reason not to do
// that, accept the speed improvement and DO NOT TOUCH the next three lines.

    if ($this->content !== NULL) {//Cr: if there is content, return it for
                                  //    display
        return $this->content;    //Cr: I feel that this sort of caching would
                                  //    work only if the ->content were a
                                  //    global. But as we have to recompute
                                  //    the mycourses part of admins and
                                  //    teachers depending on their course
                                  //    toggles, it would not help caching
                                  //    this content. And in the new merged
                                  //    version we write the contents to files
                                  //    to cache, load and recompute it as
                                  //    needed.
    }

    $this->content = new stdClass;//Cr: according to
                                  //    MoodleDocs:Development:Blocks

    $this->content->text = '';//Cr: initialize to empty string in which case
                              //    the block is not displayed
    $this->content->footer = '';//according to MoodleDocs:Development:Blocks

    if (empty ($this->instance)) {//Cr: most blocks have this code, but I can
                                  //    not imagine a block not having been
        return $this->content;    //    instantiated/added on the front or
    }                             //    any other course page

//Cr: otherwise 'compute' and fill in the content to be displayed in the block

//Cr: first 'compute' sitenavigation part, if admin set to show it, otherwise skip it

    if ($CFG->block_sitenavigation_show) {

        $this->content->text = $this->get_content_sitenavigation();
    }

//Cr: second 'compute' mycourses part, if admin set to show it, otherwise skip it

    if ($CFG->block_sitenavigation_mycourses_show) {

        $this->content->text .= '<br><hr>';//Cr: to separate from sitenavigation
        $this->content->text .= $this->get_content_mycourses();
    }

    return $this->content;
}


/**
 * Function to compose the contents of the sitenavigation-Tree
 *
 * @uses $CFG
 * @return object $this->sitenavigationcontent
 */

function get_content_sitenavigation() {
    global $CFG;//Cr: use as needed $USER, $THEME, $SESSION

//Cr: to show all courses link
    $showallcourses = get_string('showallcourses', 'block_sitenavigation');

    $this->sitenavigationcontent = new stdClass;//Cr: according to
                                                //    MoodleDocs:Development:Blocks

    $this->sitenavigationcontent->text = $this->build_navtree_sitenavigation();
    $this->sitenavigationcontent->text .= '<br>';

//Cr: show/hide the "all courses" link
    if ($CFG->block_sitenavigation_showallcourseslink) {
        $this->sitenavigationcontent->text .=
            '<center><a href="' . $CFG->wwwroot . '/course">' . $showallcourses . '</a></center>';
    }

    if ($CFG->block_sitenavigation_showsearchbox) {//Cr: added option to display
                                                   //    moodle course search box
//Cr: simply add output of moodle api function to footer:
//    print_course_search($value="sampleSearchString", //Cr: string in box
//    $return=false, //Cr: whether to echo or return text
//    $format="plain")//Cr: plain, short or navbar format
        $this->sitenavigationcontent->text .= '<center>';
        $this->sitenavigationcontent->text .= print_course_search("",true,"short");
        $this->sitenavigationcontent->text .= '</center>';
    }

    if ($CFG->block_sitenavigation_showinpopupwindow) {
//Cr: trying to show everything in a popup window instead of the block itself.
//    Actually this offers better performance because the floating window stays at the users
//    fingertips and has not to be generated on every course page visited. And even if
//    the users click on the link to pop up the window again, we check whether the file
//    with the navigation tree already exists before creating it again.

        $sitenavigationfile = '/1/sess_' . sesskey() . '_sitenavigation.html';//Cr: do not forget to purge/delete those files in a cron job

//debugging('sitenavigationFileName: ' . $CFG->dataroot . $sitenavigationfile, DEBUG_DEVELOPER);//Cr: for debugging

        if (!file_exists($CFG->dataroot . $sitenavigationfile)) {//Cr: create it first
//debugging('sitenavigationFile does not exist: ' . $CFG->dataroot . $sitenavigationfile, DEBUG_DEVELOPER);//Cr: pay attention not to show important paths
            if (!$mychandle = fopen($CFG->dataroot . $sitenavigationfile, 'w')) {//Cr: for writing, file is created

//Cr: TO DO: actually creating the file on the fly, but once in a time interval will preserve a lot of resources

debugging(get_string('couldnotopensitenavigationfile', 'block_sitenavigation'));
            }

//Cr: prepare necessary header and footer lines for correct UTF-8 display
$sitenavigationheader = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">' . "\r\n";
$sitenavigationheader .= '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">' . "\r\n";
$sitenavigationheader .= '<head>' . "\r\n";
$sitenavigationheader .= '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />'. "\r\n";
$sitenavigationheader .= '</head><body>' . "\r\n";

$sitenavigationfooter = '</body></html>' . "\r\n";

            if (!fwrite($mychandle, $sitenavigationheader . $this->sitenavigationcontent->text . $sitenavigationfooter )) {//Cr: write contents to file which serves as cache for the whole session
debugging(get_string('couldnotwritetositenavigationfile', 'block_sitenavigation'));
            }

            fclose($mychandle);//Cr: close file
        } else {
//debugging('sitenavigationFile does exist: ' . $CFG->dataroot . $sitenavigationfile, DEBUG_DEVELOPER);//Cr: pay attention not to show important paths
        }

//Cr: possible parameters for window.open in openpopup() function, no spaces after commas!:
//    height,width,left,top,resizable,scrollbars,toolbar,menubar,location,status,fullscreen,directories,copyhistory
//NOTE: if you do not pass any params, openpopup() will assign these ones:
//    $options = 'menubar=0,location=0,scrollbars,resizable';


        $this->sitenavigationcontent->text = '<a title="' . get_string('showinpopupwindow','block_sitenavigation') .
                              '" href="' . $CFG->wwwroot  . $sitenavigationfile .
                              '" target="sitenavigationPopUp" '.
                              "onclick=\"this.target='popup'; ".
                              "return openpopup('/file.php" . $sitenavigationfile ."', 'sitenavigationPopUp', ".
                              "'menubar,toolbar,location,status,scrollbars,resizable,width=1024,height=768', 0);\">" .
                              get_string('showinpopupwindow','block_sitenavigation') . "</a>";
    }

    return $this->sitenavigationcontent->text;
}



/**
 * Function to compose the contents of the mycourses-Tree
 *
 * @uses $CFG
 * @return object $this->mycoursescontent
 */

function get_content_mycourses() {
    global $CFG;//Cr: use as needed $USER, $THEME, $SESSION

//Cr: to show all courses link
    $showallcourses = get_string('showallcourses', 'block_sitenavigation');

    $this->mycoursescontent = new stdClass;//Cr: according to
                                           //    MoodleDocs:Development:Blocks
    $this->mycoursescontent->text = $this->title;//Cr: initialize title display
    
//Cr: make sure user is logged in, guest/anonymous users have no courses
    if (isloggedin()) {
        $this->mycoursescontent->text = $this->get_header_mycourses();//Cr: fill with toggles/buttons
        $this->mycoursescontent->text .= '<br><br>';
        $this->mycoursescontent->text .= $this->build_navtree_mycourses();
        $this->mycoursescontent->text .= '<br>';
    }

//Cr: show/hide the "all courses" link
    if ($CFG->block_sitenavigation_mycourses_showallcourseslink) {
        $this->mycoursescontent->text .=
            '<center><a href="' . $CFG->wwwroot . '/course">' . $showallcourses . '</a></center>';
    }

    if ($CFG->block_sitenavigation_mycourses_showsearchbox) {//Cr: added option to display
                                              //    moodle course search box
//Cr: simply add output of moodle api function to footer:
//    print_course_search($value="sampleSearchString", //Cr: string in box
//    $return=false, //Cr: whether to echo or return text
//    $format="plain")//Cr: plain, short or navbar format
        $this->mycoursescontent->text .= '<center>';
        $this->mycoursescontent->text .= print_course_search("",true,"short");
        $this->mycoursescontent->text .= '</center>';
    }

    if ($CFG->block_sitenavigation_mycourses_showinpopupwindow) {
//Cr: trying to show everything in a popup window instead of the block itself.
//    Actually this offers better performance because the floating window stays at the users
//    fingertips and has not to be generated on every course page visited. And even if
//    the users click on the link to pop up the window again, we check whether the file
//    with the navigation tree already exists before creating it again.

        $mycoursesfile = '/1/sess_' . sesskey() . '_mycourses.html';//Cr: do not forget to purge/delete those files in a cron job

//debugging('mycoursesFileName: ' . $CFG->dataroot . $mycoursesfile, DEBUG_DEVELOPER);//Cr: for debugging

            if (!file_exists($CFG->dataroot . $mycoursesfile)) {//Cr: create it first
//debugging('mycoursesFile does not exist' .  $CFG->dataroot . $mycoursesfile, DEBUG_DEVELOPER);//Cr: pay attention not to show important paths
            if (!$mychandle = fopen($CFG->dataroot . $mycoursesfile, 'w')) {//Cr: for writing, file is created
debugging(get_string('couldnotopenmycoursesfile', 'block_sitenavigation'));
            }

//Cr: prepare necessary header and footer lines for correct UTF-8 display
$mycoursesheader = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">' . "\r\n";
$mycoursesheader .= '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">' . "\r\n";
$mycoursesheader .= '<head>' . "\r\n";
$mycoursesheader .= '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />'. "\r\n";
$mycoursesheader .= '</head><body>' . "\r\n";

$mycoursesfooter = '</body></html>' . "\r\n";

            if (!fwrite($mychandle, $mycoursesheader . $this->mycoursescontent->text . $mycoursesfooter )) {//Cr: write contents to file which serves as cache for the whole session
debugging(get_string('couldnotwritetomycoursesfile', 'block_sitenavigation'));
            }

            fclose($mychandle);//Cr: close file
        } else {
//debugging('mycoursesFile does exist: ' . $CFG->dataroot . $mycoursesfile, DEBUG_DEVELOPER);//Cr: pay attention not to show important paths
        }

//Cr: possible parameters for window.open in openpopup() function, no spaces after commas!:
//    height,width,left,top,resizable,scrollbars,toolbar,menubar,location,status,fullscreen,directories,copyhistory
//NOTE: if you do not pass any params, openpopup() will assign these ones:
//    $options = 'menubar=0,location=0,scrollbars,resizable';

        $this->mycoursescontent->text = '<a title="' . get_string('mycourses_showinpopupwindow','block_sitenavigation') .
                              '" href="' . $CFG->wwwroot . '/file.php' . $mycoursesfile .
                              '" target="mycoursesPopUp" '.
                              "onclick=\"this.target='popup'; ".
                              "return openpopup('/file.php" . $mycoursesfile ."', 'mycoursesPopUp', ".
                              "'menubar,toolbar,location,status,scrollbars,resizable,width=1024,height=768', 0);\">" .
                              get_string('mycourses_showinpopupwindow','block_sitenavigation') . "</a>";
    }

    return $this->mycoursescontent->text;
}

/**
 *
 * set_user_options()
 *
 * Cr: changed name of this function from set_marker_options to
 * set_teacher_options to better reflect display options of teachers in
 * analogy with set_admin_options
 *
 * NEW: changed and unified to set_user_options() to avoid another call to
 *      a separate function. We have to set the options of students first
 *      who can only toggle old courses on and off. Then we check for
 *      admin options and finally for teacher options, the latter ones
 *      overriding the admin defaults.
 *
 * Cr: extract toggling options to show/hide given courses in
 * the $_GET response for teachers. Changed variable name mycmem to mycmyc to
 * better reflect myCourses of teachers, i.e. the courses they teach in

 *
 * @uses $SESSION
 * @var  boolean $this->mycact   to toggle display of active courses
 * @var  boolean $this->mycinact to toggle dispaly of inactive courses
 * @var  boolean $this->mycold   to toggle display of old = all courses
 * @var  boolean $this->mycmyc   to toggle display of myCourses
 *
 */

function set_user_options() {
    global $SESSION, $CFG;

    if (isset ($_GET['mycold'])) {//Cr: show/hide old = all courses
        $SESSION->mycold = ($_GET['mycold'] == 'on') ? true : false;
    } else {
        if (!isset ($SESSION->mycold)) {//Cr: if the admin option is > 0 days we do not want to show old courses
            $SESSION->mycold = ($CFG->block_sitenavigation_mycourses_notvisitedsince > 0) ? true : false;
        }
    }

    if ($this->isadmin) {//Cr: admins see ALL courses and may choose to show/hide
                         //    them in the header of the block

//Cr: mycourses added also for admins
	    if (isset ($_GET['mycmyc'])) {//Cr: show/hide all courses the admin is enrolled in
	        $SESSION->mycmyc = ($_GET['mycmyc'] == 'on') ? true : false;
	    } else {
	        if (!isset ($SESSION->mycmyc)) {//Cr: do not show them as default
	            $SESSION->mycmyc = isset ($CFG->block_mycourses_adminshowmycourses) ? $CFG->block_mycourses_adminshowmycourses : false;
	        }
	    }

	    if (isset ($_GET['mycact'])) {//Cr: show/hide all active courses
	        $SESSION->mycact = ($_GET['mycact'] == 'on') ? true : false;
	    } else {
	        if (!isset ($SESSION->mycact)) {//Cr: do not show them as default
	            $SESSION->mycact = isset ($CFG->block_mycourses_adminshowactivecourses) ? $CFG->block_mycourses_adminshowactivecourses : false;
	        }
	    }

	    if (isset ($_GET['mycinact'])) {//Cr: show/hide all inactive courses
	        $SESSION->mycinact = ($_GET['mycinact'] == 'on') ? true : false;
	    } else {
	        if (!isset ($SESSION->mycinact)) {//Cr: do not show them as default
	            $SESSION->mycinact = isset ($CFG->block_mycourses_adminshowinactivecourses) ? $CFG->block_mycourses_adminshowinactivecourses :false;
	        }
	    }
    }

    if ($this->isteacher) {//Cr: teachers see only their own courses and may choose to show/hide
                           //    them in the header of the block

	    if (isset ($_GET['mycmyc'])) {//Cr: show/hide all courses the teacher is
	                                  //    teaching in
	        $SESSION->mycmyc = ($_GET['mycmyc'] == 'on') ? true : false;
	    } else {
	        if (!isset ($SESSION->mycmyc)) {
	            $SESSION->mycmyc = true;
	        }
	    }

	    if (isset ($_GET['mycinact'])) {//Cr: show/hide all inactive courses
	        $SESSION->mycinact = ($_GET['mycinact'] == 'on') ? true : false;
	    } else {
	        if (!isset ($SESSION->mycinact)) {
	            $SESSION->mycinact = true;
	        }
	    }
    }

    $this->mycmyc = (isset ($SESSION->mycmyc)) ? $SESSION->mycmyc : true;//Cr: store settings in member variables
    $this->mycact = (isset ($SESSION->mycact)) ? $SESSION->mycact : true;//    for later display
    $this->mycinact = (isset ($SESSION->mycinact)) ? $SESSION->mycinact : true;
    $this->mycold = (isset ($SESSION->mycold)) ? $SESSION->mycold : true;

//Cr: for debugging
//print_r(array('set user_options','mycinact',$this->mycinact,'mycact',$this->mycact,'mycmyc',$this->mycmyc,'mycold',$this->mycold));

}


/** TO BE REMOVED AFTER TESTING !!
 *
 * set_teacher_options()
 *
 * Cr: changed name of next function from set_marker_options to
 * set_teacher_options to better reflect display options of teachers in
 * analogy with set_admin_options
 *
 * Cr: extract toggling options to show/hide given courses in
 * the $_GET response for teachers. Changed variable name mycmem to mycmyc to
 * better reflect myCourses of teachers, i.e. the courses they teach in
 *
 * @uses $SESSION
 * @var  boolean $this->mycact   to toggle display of active courses
 * @var  boolean $this->mycinact to toggle display of inactive courses
 * @var  boolean $this->mycold   to toggle display of old = all courses
 * @var  boolean $this->mycmyc   to toggle display of myCourses
 *
 */

function set_teacher_options() {
    global $SESSION;

    if (isset ($_GET['mycmyc'])) {//Cr: show/hide all courses the teacher is
                                  //    teaching in
        $SESSION->mycmyc = ($_GET['mycmyc'] == 'on') ? true : false;
    } else
        if (!isset ($SESSION->mycmyc)) {
            $SESSION->mycmyc = true;
        }

    if (isset ($_GET['mycinact'])) {//Cr: show/hide all inactive courses
        $SESSION->mycinact = ($_GET['mycinact'] == 'on') ? true : false;
    } else
        if (!isset ($SESSION->mycinact)) {
            $SESSION->mycinact = true;
        }

    $this->mycmyc = $SESSION->mycmyc;//Cr: store settings in member variables
    $this->mycinact = $SESSION->mycinact;//for later display
}


/**
 *
 * get_header_mycourses()
 *
 * Cr: removed custom title hack, which was not clear any more. But see
 * the comments I made in the code if you want this functionality back:
 * http://moodle.org/mod/forum/discuss.php?d=16076, post of Daryl Hawes.
 *
 * The function composes the title of the block and returns what is needed
 * for admins, teachers, and normal users, i.e. the simple title string or
 * a table containing two icons and links to toggle the display of
 * active, inactive and my courses for admins and teachers.
 *
 * NEW: added the green icon oldon.gif and oldoff.gif to display actually
 *      attended courses or all courses where the user is enrolled.
 *
 * @note This function toggles the display sending a GET-Request which causes
 *       the whole page to be rebuilt by the server, which is very inefficient.
 *       A better approach would probably involve some javascript programming
 *       to simply toggle what the TreeMenu should display.
 *
 * @uses $CFG
 * @uses $FULLME
 *
 * @var string iurl to store url of inactive courses
 * @var string aurl to store url of active courses
 * @var string murl to store url of myCourses of teachers/admins
 * @var string ourl to store url of old = all courses
 * @var string iimg to store icon of inactive courses
 * @var string aimg to store icon of active courses
 * @var string mimg to store icon of myCourses of teachers/admins
 * @var string oimg to store icon of old = all courses
 *
 * @return object HTML-Text/Table to be displayed as Title-Bar of the block
 */

function get_header_mycourses() {
    global $CFG, $FULLME;//Cr: not needed any more, $SESSION;

    $iurl = preg_replace('/([\&\?]mycact=(on|off)|[\&\?]mycinact=(on|off)|[\&\?]mycmyc=(on|off)|[\&\?]mycold=(on|off))/', '', $FULLME);
    $aurl = $iurl;
    $murl = $iurl;
    $ourl = $iurl;

    $iimg = '';
    $aimg = '';
    $mimg = '';
    $oimg = '';

    $ititle = '';
    $atitle = '';
    $mtitle = '';
    $otitle = '';

    if ($this->mycold) {//Cr: compose table with icons and
    	                //    urls to toggle display of given courses
        $oimg = $CFG->wwwroot . '/blocks/sitenavigation/pix/oldon.gif';
        $otitle = ' title="' . get_string('mycourses_hideoldcourses', 'block_sitenavigation') . '."';
        if (strpos($ourl, '?')) {
            $ourl .= '&mycold=off';
        } else {
            $ourl .= '?mycold=off';
        }
    } else {
        $oimg = $CFG->wwwroot . '/blocks/sitenavigation/pix/oldoff.gif';
        $otitle = ' title="' . get_string('mycourses_showoldcourses', 'block_sitenavigation') . '."';
        if (strpos($iurl, '?')) {
            $ourl .= '&mycold=on';
        } else {
            $ourl .= '?mycold=on';
        }
    }
//Cr: next line to be removed to show icons to all users now, including students
    if ($this->isadmin || $this->isteacher) {//Cr: compose table with icons and
        if ($this->mycinact) {               //    urls to toggle display of given courses
            $iimg = $CFG->wwwroot . '/blocks/sitenavigation/pix/inacton.gif';
            $ititle = ' title="' . get_string('mycourses_hideinactivecourses', 'block_sitenavigation') . '."';
            if (strpos($iurl, '?')) {
                $iurl .= '&mycinact=off';
            } else {
                $iurl .= '?mycinact=off';
            }
        } else {
            $iimg = $CFG->wwwroot . '/blocks/sitenavigation/pix/inactoff.gif';
            $ititle = ' title="' . get_string('mycourses_showinactivecourses', 'block_sitenavigation') . '."';
            if (strpos($iurl, '?')) {
                $iurl .= '&mycinact=on';
            } else {
                $iurl .= '?mycinact=on';
            }
        }

       //Cr: next section only for admins, because only admins can see all courses
        if ($this->isadmin) {
            if ($this->mycact) {
                $aimg = $CFG->wwwroot . '/blocks/sitenavigation/pix/acton.gif';
                $atitle = ' title="' . get_string('mycourses_hideactivecourses', 'block_sitenavigation') . '."';
                if (strpos($aurl, '?')) {
                    $aurl .= '&mycact=off';
                } else {
                    $aurl .= '?mycact=off';
                }
            } else {
                $aimg = $CFG->wwwroot . '/blocks/sitenavigation/pix/actoff.gif';
                $atitle = ' title="' . get_string('mycourses_showactivecourses', 'block_sitenavigation') . '."';
                if (strpos($aurl, '?')) {
                    $aurl .= '&mycact=on';
                } else {
                    $aurl .= '?mycact=on';
                }
            }

        }

        if ($this->mycmyc) {
            $mimg = $CFG->wwwroot . '/blocks/sitenavigation/pix/mycon.gif';
            $mtitle = ' title="' . get_string('mycourses_hidemycourses', 'block_sitenavigation') . '."';
            if (strpos($murl, '?')) {
                $murl .= '&mycmyc=off';
            } else {
                $murl .= '?mycmyc=off';
            }
        } else {
            $mimg = $CFG->wwwroot . '/blocks/sitenavigation/pix/mycoff.gif';
            $mtitle = ' title="' . get_string('mycourses_showmycourses', 'block_sitenavigation') . '."';
            if (strpos($murl, '?')) {
                $murl .= '&mycmyc=on';
            } else {
                $murl .= '?mycmyc=on';
            }
        }
    }
//Cr: return title embedded in a table with GUI to toggle show/hide teachers's
//    and admin's courses

//Cr: actually only in block-display until we have a further admin option for this    

//Cr: actually only in block-display until we have a further admin option for this

if (!$CFG->block_sitenavigation_mycourses_showinpopupwindow) {

        return '<table cellpadding="0" cellspacing="0" border="0"><tr>' .
        '<td width="100%"><b>' .  get_string('mycourses_blocktitle','block_sitenavigation') . '</b></td>' .
        '<td align="right" nowrap>' .
         (!empty ($mimg) ? '<a href="' . $murl . '"' . $mtitle . '><img src="' . $mimg . '" /></a>' : '') .
         (!empty ($aimg) ? '<a href="' . $aurl . '"' . $atitle . '><img src="' . $aimg . '" /></a>' : '') .
         (!empty ($iimg) ? '<a href="' . $iurl . '"' . $ititle . '><img src="' . $iimg . '" /></a>' : '') .
         (!empty ($oimg) ? '<a href="' . $ourl . '"' . $otitle . '><img src="' . $oimg . '" /></a>' : '') .
        '</td></tr></table>';
} else {
	   return '<b>' . get_string('mycourses_blocktitle','block_sitenavigation') . '</b>';//Cr: return title string as is
}    
////Cr: end of block-display

/*
    } else {
        return '<b>' . get_string('mycourses_blocktitle','block_sitenavigation') . '</b>';//Cr: return title string as is for normal users
    }
*/
}


/**
 *
 * build_navtree_sitenavigation() constructs a hierarchical navigation tree with icons of open/closed folders
 * and returns it
 *
 * Cr: changed function name from build_menu to build_navtree to better reflect what the
 * function does
 *
 * @uses $CFG
 * @uses $USER
 * @uses $SITE
 *
 * @var object $this->navTree_sitenavigation the whole navigation tree
 * @var boolean $adminseesall can be used to show own or all courses for admins
 * @var array $categories for the cats returned by get_categories()
 * @var array $this->cattree_sitenavigation to hold the tree of categories returned by $this->load_cattree($categories)
 * @var array $courses to hold courses returned by get_my_courses or get_records('course')
 *
 * @return object $output
 */

function build_navtree_sitenavigation() {
    global $CFG, $USER, $SITE;

    $this->navtree_sitenavigation = new HTML_Tree();//Cr: changed name of member variable
//from $this->menu to $this->navTree to better reflect what is stored here.
//Cr: try passing this param: array('linkTarget' => 'sitenavigation')

/// Build a tree of categories for later use.
    $categories = get_categories(0);
//print_r(array('categories: ', $categories));//Cr: for debugging

//fast hack to load all courses/categories also for not logged in users
//$courses = get_courses("all", "c.sortorder ASC", "c.*");
//$courses = get_records('course', 'category', '21');//try this to load courses of only one given category

    $this->cattree_sitenavigation = $this->load_cattree($categories);//Cr: search recursively
//through all cats and save a reference to that tree
    $output = '';//Cr: initialize to empty

//Cr: make sure user is logged in
    if ($CFG->block_sitenavigation_loginrequired && !isloggedin()) {
//print_r(array('loginRequired',' but user is not logged in'));//Cr: for debugging
      $courses = array();//Cr: empty array
      return $output;    //    and we could jump out to display nothing, not
    }                    //    even the block
    else {
//print_r(array('loginNotRequired or user is logged in',' both see the navigation'));//Cr: for debugging

//Cr: Trick to show only categories which load much faster than thousands of
//courses. See also create_tree_menu below
      if ($CFG->block_sitenavigation_sitenavigationbycategory) {
//print_r(array('siteNavByCategories',' only categories are displayed'));//Cr: for debugging
        $courses = $categories;
      }
      else {
//print_r(array('siteNavByCategories Not set',' categories AND courses are displayed'));//Cr: for debugging
        $courses = get_records('course');//Cr: get all courses. ATTENTION,
//this may delay later HTML-rendering if you have many courses! I will look
//out for faster code, e.g. in the site admin block.

//Cr: add mNet courses here, PURELY EXPERIMENTAL until thoroughly tested
        $remotecourses = $this->get_remote_courses();
        if (!empty($remotecourses)) {
//print_r(array('not empty remotecourses',$remotecourses));//Cr: for debugging
            $result = array_merge($courses,$remotecourses);//Cr: has to be tested
            $courses = $result;//Cr: has to be tested
        }

/// First, load all of the courses into an array of categories.
        foreach ($courses as $course) {

            if (!$course->category) {//Cr: the only course belonging to
                             //category 0 is the front page, so skip it
            continue;
          }

        $this->has_courses($course);//Cr: mark all categories having courses if
                                    //the according courses are to be displayed
        }
      }
    }

    if ($this->course->id == $SITE->id) {//Cr: if we are on the front page,
                                         //    show node text in bold
//Cr: title= will show up a tool tip text on mouse hover
/* this code unfortunately does not work, maybe it should be integrated into the javascript library
        $url = $CFG->wwwroot .
        '" onClick="open_link_in_window(' . "'$CFG->wwwroot'" . ', ' . "'sitenavigation'" . ');return false" title="' .
        htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES);
*/
        $url = $CFG->wwwroot . '" title="' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES);

        if ($CFG->block_sitenavigation_showlongnames) {
            $text = ' <span style="font-weight: bold;" title="' . htmlspecialchars(format_string($SITE->shortname), ENT_QUOTES) .
            '">' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES) . '</span>';
        } else {
            $text = ' <span style="font-weight: bold;" title="' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES) .
            '">' . htmlspecialchars(format_string($SITE->shortname), ENT_QUOTES) . '</span>';
        }
        $cssclass = 'treeMenuBold';
        $icon = 'home.gif';
        $expandedIcon = 'home.gif';
    } else {//Cr: else show it in plain
        $url = $CFG->wwwroot.'" title="' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES);

        if ($CFG->block_sitenavigation_showlongnames) {
            $text = ' <span style="font-weight: plain;" title="' . htmlspecialchars(format_string($SITE->shortname), ENT_QUOTES) .
            '">' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES) . '</span>';
        } else {
            $text = ' <span style="font-weight: plain;" title="' . format_string($SITE->fullname) .
            '">' . htmlspecialchars(format_string($SITE->shortname), ENT_QUOTES) . '</span>';
        }
        $cssclass = 'treeMenuDefault';
        $icon = 'home.gif';
        $expandedIcon = 'home.gif';
    }

//Cr: linkTarget added to open links to the Moodle pages in the same/new window/tab
    $mnode = new HTML_TreeNode1(array(
        'text' => $text,
        'link' => $url,
        'icon' => $icon,
        'cssClass' => $cssclass,
        'expandedIcon' => $expandedIcon,
        'linkTarget' => 'sitenavigation',
        'expanded' => true
    ));
    $this->create_tree_menu_sitenavigation($this->cattree_sitenavigation, $mnode);
    $this->navtree_sitenavigation->addItem($mnode);

//Cr: linkTarget added to open links to the Moodle pages in the same/new window/tab
    $treemenu = &new HTML_Tree_DHTML($this->navtree_sitenavigation, array(
        'images' => $CFG->wwwroot.'/blocks/sitenavigation/HTML_TreeMenu-sitenavigation/images',
        'linkTarget' => 'sitenavigation', 'defaultClass' => 'treeMenuDefault'
    ));

    ob_start();
    $treemenu->printMenu();
    $output = '
		<style type="text/css">
			.treeMenuDefault {
				font-size: 90%;
				font-style: normal;
			}

			.treeMenuBold {
				font-size: 90%;
				font-weight: bold;
			}
		</style>';

    $output .= '<script src="'.$CFG->wwwroot.'/blocks/sitenavigation/HTML_TreeMenu-sitenavigation/Tree.js" language="JavaScript" type="text/javascript"></script>';
    $output .= ob_get_contents();
    ob_end_clean();

    return $output;
}



/**
 *
 * build_navtree_mycourses() constructs a hierarchical navigation tree with icons of
 * open/closed folders and returns it
 *
 * Cr: changed function name from build_menu to build_navtree to better
 * reflect what the function does
 *
 * Cr: NOTE that this function loads the needed courses and calls
 * the function has_courses_mycourses to filter what courses are actually
 * displayed, i.e. mycourses for teachers and students (or all
 * courses for admins), active and/or inactive courses
 *
 * @uses $CFG
 * @uses $USER
 * @uses $SITE
 *
 * @var object $this->navTree_mycourses the whole navigation tree
 * @var boolean $adminseesall can be used to show own or all courses for admins
 * @var array $categories for the cats returned by get_categories()
 * @var array $this->cattree_mycourses to hold the tree of categories returned by $this->load_cattree($categories)
 * @var array $courses to hold courses returned by get_my_courses or get_records('course')
 *
 * @return object $output
 */

function build_navtree_mycourses() {
    global $CFG, $USER, $SITE;

    $this->navTree_mycourses = new HTML_TreeMenu();//Cr: changed name of member variable
//from $this->menu to $this->navTree to better reflect what is stored here.
//Cr: test passing this param: array('linkTarget' => 'mycourses')

/// Build a tree of categories for later use.
    $categories = get_categories(0);

//print_r($categories);//Cr: for debugging

    $this->cattree_mycourses = $this->load_cattree($categories);//Cr: search recursively
//through all cats and save a reference to that tree

//Cr: in next line we have adjusted the logic to display teacher's and admin's
//active, inactive and mycourses

//Cr: in a future development we could retrieve a shorter mycourses list for
//    admins if we use capability and the function call get_user_course_bycap
// like this
// $accessinfo = get_user_access_sitewide($userid);
// $doanything = false;
// $courses = get_user_courses_bycap($userid, 'moodle/course:view', $accessinfo,
//                                        $doanything, $sort, $fields,
//                                        $limit);
// but we have to decide first on which capability/roles we want to check
// for admins enrolled in courses, e.g. as students, or in own created roles
// See also the function has_courses_mycourses below which filters the courses to display

//Cr: I am using an own copy of get_my_courses here because it checks also for
//    courses in hidden categories in stead of using get_user_courses_bycap as
//    mentioned above.

    $courses = array();//Cr: initialize variable for safety

    if ($this->isadmin) {//Cr: check for all admin cases
        if ($this->mycmyc) {
//Cr: get mycourses of Admins if they toggled to display them. Feel free to add
//    as many other capabilities/roles as you want.

//Cr: for debugging
//print_r(array('ADMIN wants to see his own courses: ',$USER->id,'myc',$this->mycmyc,'mycact',$this->mycact,'mycinact',$this->mycinact));

            if (!($coursesteacheredit = $this->get_my_courses_bycap($USER->id, 'moodle/legacy:editingteacher'))) {
                $coursesteacheredit = array ();
            }
            if (!($coursesteacher = $this->get_my_courses_bycap($USER->id, 'moodle/legacy:teacher'))) {
                $coursesteacher = array ();
            }
            if (!($coursesstudent = $this->get_my_courses_bycap($USER->id, 'moodle/legacy:student'))) {
                $coursesstudent = array ();
            }
            $courses = array_merge($coursesteacheredit, $coursesteacher, $coursesstudent);

//Cr: unfortunately the next line produces an error as "Array ( [0] => stdClass Object(" contained in the arrays
//    can not be converted to a string. For the moment we leave the duplicates if any
            //$courses = array_unique($courses);//Cr: show courses only once if admin enrolled with different roles

        } else {
           if (!$this->mycact && !$this->mycinact) {
//Cr: the Admin user dose not want to see any courses
//Cr: for debugging
//print_r(array('ADMIN does not want to see any courses: ',$USER->id,'myc',$this->mycmyc,'mycact',$this->mycact,'mycinact',$this->mycinact));
               $courses = array ();
            }else {
//Cr: admin wants to see all courses, either active and/or inactive ones
//Cr: for debugging
//print_r(array('ADMIN wants to see all courses: ',$USER->id,'myc',$this->mycmyc,'mycact',$this->mycact,'mycinact',$this->mycinact));
                if (!($courses = get_records('course'))) {
                    $courses = array ();
                }
            }
        }
    } else {//Cr: check for all teacher cases
        if ($this->isteacher) {
//Cr: get mycourses of teachers if they toggled to see them
            if ($this->mycmyc) {
//Cr: for debugging
//print_r(array('load only mycourses of teacher: ',$USER->id,'myc',$this->mycmyc,'mycact',$this->mycact));
                if (!($courses = get_my_courses($USER->id))) {
                    $courses = array ();
                }
            } else {//Cr: teacher does not want to see any courses
//Cr: for debugging
//print_r(array('TEACHER does not want to see any courses: ',$USER->id,'myc',$this->mycmyc,'mycinact',$this->mycinact));
                $courses = array ();
            }
        } else {//Cr: no cases to check for students, simply load their courses
//Cr: for debugging
//print_r(array('load only mycourses of students: ',$USER->id));
            if (!($courses = get_my_courses($USER->id))) {
                $courses = array ();
            }
        }
    }

//Cr: add mNet courses here, PURELY EXPERIMENTAL until thoroughly tested
	$remotecourses = $this->get_remote_courses();
	if (!empty($remotecourses)) {
//print_r(array('not empty remotecourses',$remotecourses));//Cr: for debugging
		$result = array_merge($courses,$remotecourses);//Cr: has to be tested
		$courses = $result;//Cr: has to be tested
	}

//print_r($courses);//Cr: for debugging

/// First, load all of the courses into an array of categories.
    foreach ($courses as $course) {

        if (!$course->category) {//Cr: the only course belonging to category 0
                                 //    is the front page, so skip it
            continue;
        }

        $this->has_courses_mycourses($course);//Cr: mark all categories having courses if
                                    //the according courses are to be displayed
    }

    if ($this->course->id == $SITE->id) {//Cr: if we are on the front page,
                                         //    show node text in bold
//Cr: title= will show up a tool tip text on mouse hover
        $url = $CFG->wwwroot . '" title="' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES);
        if ($CFG->block_sitenavigation_mycourses_showlongnames) {
            $text = ' <span style="font-weight: bold;" title="' . htmlspecialchars(format_string($SITE->shortname), ENT_QUOTES) .
            '">' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES) . '</span>';
        } else {
            $text = ' <span style="font-weight: bold;" title="' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES) .
            '">' . htmlspecialchars(format_string($SITE->shortname), ENT_QUOTES) . '</span>';
        }
        $cssclass = 'treeMenuBold';
        $icon = 'home.gif';
        $expandedicon = 'home.gif';
    } else {//Cr: else show it in plain
        $url = $CFG->wwwroot . '" title="' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES);
        if ($CFG->block_sitenavigation_mycourses_showlongnames) {
            $text = ' <span style="font-weight: plain;" title="' . htmlspecialchars(format_string($SITE->shortname), ENT_QUOTES) .
            '">' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES) . '</span>';
        } else {
            $text = ' <span style="font-weight: plain;" title="' . htmlspecialchars(format_string($SITE->fullname), ENT_QUOTES) .
            '">' . htmlspecialchars(format_string($SITE->shortname), ENT_QUOTES) . '</span>';
        }
        $cssclass = 'treeMenuDefault';
        $icon = 'home.gif';
        $expandedicon = 'home.gif';
    }

//Cr: linkTarget added to display Moodle pages always in a same/new window/tab
    $mnode = new HTML_TreeNode(array (
        'text' => $text,
        'link' => $url,
        'icon' => $icon,
        'cssClass' => $cssclass,
        'expandedIcon' => $expandedicon,
        'linkTarget' => 'mycourses',
        'expanded' => true
    ));
    $this->create_tree_menu_mycourses($this->cattree_mycourses, $mnode);
    $this->navTree_mycourses->addItem($mnode);

//Cr: linkTarget added to display Moodle pages alsways in a same/new window/tab
    $treemenu = & new HTML_TreeMenu_DHTML($this->navTree_mycourses, array (
        'images' => $CFG->wwwroot . '/blocks/sitenavigation/HTML_TreeMenu-mycourses/images',
        'linkTarget' => 'mycourses', 'defaultClass' => 'treeMenuDefault'
    ));

    ob_start();
    $treemenu->printMenu();
    $output = '
        <style type="text/css">
            .treeMenuDefault {
                font-size: 90%;
                font-style: normal;
            }

            .treeMenuBold {
                font-size: 90%;
                font-weight: bold;
            }
        </style>';

    $output .= '<script src="' . $CFG->wwwroot . '/blocks/sitenavigation/HTML_TreeMenu-mycourses/TreeMenu.js" language="JavaScript" type="text/javascript"></script>';
    $output .= ob_get_contents();
    ob_end_clean();

    return $output;
}

/**
 * Compose indented tree recursively and return it
 *
 *Cr: It seems, this function is not used any more, I guess the HTMLtreeMenu
 *    does the needed indentation. I will remove that code on next cleanup.
 *
 * @param array $branches
 * @param object $output
 * @param integer $indent default is 0
 * @return object $output


function print_menu($branches, $output='', $indent=0) {

    foreach ($branches as $node) {
        if (!empty($output)) {
            $output .= '<br />';
        }
        if ($indent) {
            for ($i=0; $i<$indent; $i++) {
                $output .= '&nbsp;';
            }
        }
        $output .= $node->node->name;
        if ($node->branches) {
            $output = $this->print_menu($node->branches, $output, $indent+2);
        }
    }
    return $output;
}
*/


/**
 * Compose Tree of categories recursively and return it
 *
 * @param array &$categories
 * @return array $categories
 */

function load_cattree(&$categories) {

//print_r(array('categories',$categories));//Cr: for debugging
//print_r(array('next recursive call'));//Cr: for debugging


//Cr: this loop and recursive calls have to be changed, so as to use only
//    a simple query to retrieve all needed data and to populate the
//    $categories data structure.
//    The question arises anyway if looping through a complex data structure in
//    memory will be worth the extra work here or if it would be better to
//    schedule the whole code once, twice a day, or even every 15 minutes and
//    save the output into a file. Then we would
//    let all the instances of sitenavigation load the resulting code from
//    that file, which will then yield a significant speed improvement, as
//    on every course-page where the block is used would not recreate the
//    tree on the fly but from the simple file. Even in the merged version of
//    mycourses and sitenavigation we could load the "static" file and compose
//    the individual part of the user's navigation, typically his own courses
//    on the fly.

    foreach ($categories as $category) {
        if ($cats = get_categories($category->id)) {
            $categories[$category->id]->categories = $this->load_cattree($cats);
        } else {
            $categories[$category->id]->categories = array();
        }
        $categories[$category->id]->hascourses = false;
        $this->categories[(int)$category->id] = &$categories[$category->id];
    }

//print_r(array('categories',$categories));//Cr: for debugging

    return $categories;
}

/**
 * Function that loads the courses [applicable to the current selected
 * filter settings.]
 *
 * Cr: the filter settings apply only for mycourses, we keep only
 *     what we need to correctly display the categories/courses
 *
 * @uses $CFG
 * @param object $course The course to check against.
 * @return boolean
 */

function has_courses($course) {
    global $CFG;
//Load this course for display if it's not filtered away by any admin toggles

//If it's an invisible course, and block_sitenavigation_showhiddencourses is false, don't show it

    if ( !$course->visible && !$CFG->block_sitenavigation_showhiddencourses ) {

//Cr: for debugging
//print_r(array($course, 'do not show inactive courses'));
	    return;
    }

    $this->categories[$course->category]->hascourses = true;

//Cr: trying to sort on fullname instead of sortorder
//BUT NOTE THAT sorting takes place only inside the category, so if you display a flat list
//it will not seem to be globally sorted correctly.
//    $this->categories[$course->category]->courses[$course->sortorder] = $course;
    $this->categories[$course->category]->courses[$course->fullname] = $course;

/// Make sure the courses are in the order specified.
//Cr: for debugging
//print_r($this->categories[$course->category]->courses);

    ksort($this->categories[$course->category]->courses);

    $catid = $course->category;
    while ($this->categories[$catid]->parent > 0) {
        $catid = $this->categories[$catid]->parent;
        $this->categories[$catid]->hascourses = true;
    }
}


/**
 * Function that loads the courses applicable to the current selected
 * filter settings.
 *
 * @uses $USER
 * @param object $course The course to check against.
 * @return boolean
 */

function has_courses_mycourses($course) {
global $CFG, $USER;
//Load this course for display if it's not filtered away by admin and teacher toggles

//Cr: for debugging to see what capabilities the user has got
//print_r($USER->access); exit;

    $context = get_context_instance(CONTEXT_COURSE, $course->id);//Cr: we need it to check teachers

//CR: Using capabilities now. Note that for the moment being the logic is rather messy
//here and has to be rewritten. As a side-effect introducing capabilities here
//shows an intresting thing: if the teachers toggle off mycourses and inactive
//courses, they get to see courses in which they are enrolled as STUDENTS or
//NON-editing-Teachers. So in the merged Version of mycourses and sitenavigation
//we should probably display different sections like this for every user:
//Enrolled as TEACHER:
// - course 1
// - course 2
//Enrolled as STUDENT:
// - course 1
// - course 2
//Enrolled as NON-editing-Teacher:
// - course 1
// - course 2
//and so on and add a section for admins:
//ADMIN (all courses)
// - course 1
// - course 2
//and so on from the first to the last course.
//The single sections are collapsible and so every one may choose to expand and
//display the courses they want to, on the fly.

//Cr: Starting with Version 1.9.07a this side effect has been removed because
//    we load the admin courses using capabilities in build_navtree_mycourses. I
//    adjusted the logic to display nothing if all buttons/toggles are off
//    and to dipslay mycourses (active or inactive) ones depending on the
//    mycourses button/toggle which acts like a main toggle now.


//If it's a visible course, and the active courses filter is off, don't show it
//Cr: this case only applies for admins. Teachers are not able to see all
//    active courses.

    if ($course->visible && $this->isadmin && !$this->mycact) {

//Cr: for debugging
//print_r(array($course, 'do not show active courses of admins'));
        return;
    } else

//If it's an inactive course, and the inactive course filter is off,
//don't show it.
        if (!$course->visible && ($this->isadmin ||
            has_capability('moodle/legacy:editingteacher', $context, $USER->id, false)) &&
            !$this->mycinact) {

//Cr: for debugging
//print_r(array($course, 'do not show inactive courses of teachers or admins'));
            return;
        } else

//If it's a mycourse, and the mycourse filter is off and it is an admin
//don't show it.

//Cr: only teachers AND ADMINS can toggle mycourses on and off. But the logic
//may still be buggy here if the admin has teacher AND admin role in one
//course. In the best case he is admin which takes precedence here, and all
//courses are displayed, i.e. none is skipped. The other way round it should
//work fine because an editing teacher should normally NOT be an admin and
//hence no courses should be skipped here.

//Cr: unfortunately since 20091109 isteacheredit returns true both for teachers and admins
//    so I had to correct this here with single capabilities. Maybe we should check for more
//    capabilities in future, depending on the roles you enrol in your courses with your Moodle
//    admin account. ANYWAY I feel it to be a bad idea to enrol into courses with your
//    Moodle admin account, you would better use separate accounts for this. And if you have
//    Admins for single categories, they should use also a separate admin accounts and not
//    the one they use to enrol as teachers or students.

//Cr: here is code to check also roles rather than capabilities:
//if ($roles = get_roles_with_capability('moodle/legacy:guest', CAP_ALLOW)) {
//              $guestrole = array_shift($roles);   // Pick the first one
//


            if ( ! has_capability('moodle/legacy:editingteacher', $context, $USER->id, false) &&
                 ! has_capability('moodle/legacy:student', $context, $USER->id, false) &&
                 $this->mycmyc && $this->isadmin ) {

//Cr: for debugging
//print_r(array('testing', 'do not show courses you do not teach in for admins'));
                return;
            } else

//If it's a mycourse, and the mycourse filter is off and it is a teacher
//don't show it.

                if ( $course->visible && has_capability('moodle/legacy:editingteacher', $context, $USER->id, false) &&
                 !$this->mycmyc ) {
//Cr: for debugging
//print_r(array('testing', 'do not show courses you do not teach in for teachers'));
                    return;
                }

//Cr: trying to toggle on/off old courses. Old courses are considered as not
//    visited during a given number of days, as set by the admin option notvisitedsince

//Cr: if user does not want to see the old courses and notvisitedsince is greater than 0, filter/hide/skip those courses
//    for teachers and admins show own courses must be on otherwise it does not make sense
//    hmm, if not the toggle works also for all courses not visited, not only the own ones
//    so include this in the if-clause: $this->mycmyc && if you want it to work only on own courses
    if (!$this->mycold && $CFG->block_sitenavigation_mycourses_notvisitedsince > 0 ) {
        $timestamp = time() - $CFG->block_sitenavigation_mycourses_notvisitedsince * 86400;//Cr: calculate timestamp - number of days in seconds from today back

        $lastaccess = get_field('user_lastaccess', 'timeaccess', 'courseid',$course->id, 'userid', $USER->id);
//Cr: for debugging
//print_r(array('courseid and lastaccess: ',$course->id, $lastaccess));
        if ($lastaccess < $timestamp) {//Cr: skip this course
        	return;
        }
    }        
                
//Cr: for debugging
//print_r(array('for index and sortorder', $course));


    $this->categories[$course->category]->hascourses = true;

//Cr: trying to sort on fullname instead of sortorder
//BUT NOTE THAT sorting takes place only inside the category, so if you display a flat list
//it will not seem to be globally sorted correctly.
//    $this->categories[$course->category]->courses[$course->sortorder] = $course;
    $this->categories[$course->category]->courses[$course->fullname] = $course;

/// Make sure the courses are in the order specified.
//Cr: for debugging
//print_r($this->categories[$course->category]->courses);

    ksort($this->categories[$course->category]->courses);

    $catid = $course->category;
    while ($this->categories[$catid]->parent > 0) {
        $catid = $this->categories[$catid]->parent;
        $this->categories[$catid]->hascourses = true;
    }
}

/**
 * Create whole hierarchical Tree
 *
 * @uses $CFG
 * @param array $categories
 * @param object &$pnode
 * @return void
 */

function create_tree_menu_sitenavigation($categories, &$pnode) {
    global $CFG;

    $ficon = 'folder.gif';//Cr: changed name of variable from nicon to ficon
                          //    to better reflect its content
    $feicon = 'folder-expanded.gif';//Cr: changed name of variable from eicon
                                    //to feicon to better reflect it's content
    $cicon = 'course.gif';
    $cacticon = 'courseact.gif';
    $cinacticon = 'courseinact.gif';
    $cmycicon = 'coursemyc.gif';//Cr: for later use if myC and siteNav are merged

    foreach ($categories as $catid => $catnode) {
//Cr: do not skip those categories, so we can show them with links for
//    navigation
//if (!$catnode->hascourses) continue;

        $linkcss = '';//Cr: possible use: $linkcss = $course->visible ? '' : ' class="dimmed" ';
        $cssclass = 'treeMenuDefault';

//Cr: skip folders/categories to show only a flat list of courses with the
//    else section below
        if (!$CFG->block_sitenavigation_showflatlist) {//Cr: display with folders/cats
            if ($CFG->block_sitenavigation_showcategorylinks) {//Cr: show/hide
//link. Categories have no fullname field
                $url = $CFG->wwwroot . '/course/category.php?id=' . $catnode->id .
                '" title="' . htmlspecialchars(format_string($catnode->name), ENT_QUOTES) . $linkcss;
            } else {
                $url = '';//Cr: empty url, i.e. no link to Moodle category page
            }

            if (!empty ($this->selectedcat) && ($this->selectedcat == $catnode->id)) {
                $cssclass = 'treeMenuDefault highlight';
            }

//Cr: linkTarget added to display Moodle pages always in a same/new window/tab
            $node = & $pnode->addItem(new HTML_TreeNode(array (
                'text' => ' ' . format_string($catnode->name),
                'link' => $url,
                'icon' => $ficon,
                'expandedIcon' => $feicon,
                'linkTarget' => 'sitenavigation',
                'cssClass' => $cssclass
            )));
        } else {             //Cr: without folders/cats: add an empty node
            $node = & $pnode;//Cr: this seems to be a little hack, as we do not
                             //    create a new node here with ->addItem but
                             //    simply pass the same node's address,
        }                    //    nevertheless this should be ok, as we add
                             //    several entries to the same node

        if (!empty($catnode->categories)) {
            $this->create_tree_menu_sitenavigation($catnode->categories, $node);//Cr: recursively create nested nodes
        }

        if (!$CFG->block_sitenavigation_sitenavigationbycategory) {//Cr: the next section does not
                                                                   //    apply for categories
          if (!empty($catnode->courses)) {
            foreach ($catnode->courses as $course) {

                if (isteacheredit($course->id) ) {//Cr: display active and
                                                  //inactive courses for teachers with different icons
                    if ($course->visible) {
                        $linkcss = '';//Cr: possible use: $linkcss = $course->visible ? '' : ' class="dimmed" ';
                        $icon = $cacticon;
                    } else {
                        $linkcss .= '';//Cr: possible use: $linkcss = $course->visible ? '' : ' class="dimmed" ';
                        $icon = $cinacticon;
                    }
                } else {
                    $linkcss .= '';//Cr: possible use: $linkcss = $course->visible ? '' : ' class="dimmed" ';
                    $icon = $cicon;
                }

				if ($CFG->block_sitenavigation_showlongnames) {//Cr: show long name and short in tool tip text
					$url = $CFG->wwwroot . '/course/view.php?id=' . $course->id .
				   '" title="' . htmlspecialchars(format_string($course->shortname), ENT_QUOTES) . $linkcss;
				   $text = format_string($course->fullname);
				} else {//Cr: show short name and long in tool tip text
					$url = $CFG->wwwroot . '/course/view.php?id=' . $course->id .
				   '" title="' . htmlspecialchars(format_string($course->fullname), ENT_QUOTES) . $linkcss;
				   $text = format_string($course->shortname);
				}

                if ($this->course->id == $course->id) {//Cr: if this is the actually displayed course page
                    $url = '';//Cr: show no link and no tool tip
                    $cssclass = 'treeMenuBold';
                } else {
                    $cssclass = 'treeMenuDefault';
                }

//Cr: linkTarget added to display Moodle pages always in a same/new window/tab
                $node->addItem(new HTML_TreeNode(array(
                    'text' => $text,
                    'link' => $url,
                    'icon' => $icon,
                    'linkTarget' => 'sitenavigation',
                    'cssClass' => $cssclass
                )));
            }
          }
        }
    }
}



/**
 * Create whole hierarchical Tree for mycourses
 *
 * @uses $CFG
 * @param array $categories
 * @param object &$pnode
 * @return void
 */

function create_tree_menu_mycourses($categories, & $pnode) {
    global $CFG;

    $ficon = 'folder.gif';//Cr: changed name of variable from nicon to ficon
                          //    to better reflect its content
    $feicon = 'folder-expanded.gif';//Cr: changed name of variable from eicon
                                    //to feicon to better reflect it's content
    $cicon = 'course.gif';
    $cacticon = 'courseact.gif';
    $cinacticon = 'courseinact.gif';
    $cmycicon = 'coursemyc.gif';

    foreach ($categories as $catid => $catnode) {
        if (!$catnode->hascourses)
            continue;

        $linkcss = '';//Cr: possible use: $linkcss = $course->visible ? '' : ' class="dimmed" ';
        $cssclass = 'treeMenuDefault';

//Cr: skip folders/categories to show only a flat list of courses with the
//    else section below
        if (!$CFG->block_sitenavigation_mycourses_showflatlist) {//Cr: display with folders/categories
            if ($CFG->block_sitenavigation_mycourses_showcategorylinks) { //Cr: show/hide link
//categories have no fullname field
                $url = $CFG->wwwroot . '/course/category.php?id=' . $catnode->id .
                '" title="' . htmlspecialchars(format_string($catnode->name), ENT_QUOTES) . $linkcss;
            } else {
                $url = '';//Cr: empty url, i.e. no link to Moodle category page
            }

            if (!empty ($this->selectedcat) && ($this->selectedcat == $catnode->id)) {
                $cssclass = 'treeMenuDefault highlight';
            }

//Cr: linkTarged added to display Moodle pages always in a same/new window/tab
            $node = & $pnode->addItem(new HTML_TreeNode(array (
                'text' => ' ' . format_string($catnode->name),
                'link' => $url,
                'icon' => $ficon,
                'expandedIcon' => $feicon,
                'linkTarget' => 'mycourses',
                'cssClass' => $cssclass
            )));
        } else {             //Cr: diplay without folders/cats: add an empty node
            $node = & $pnode;//Cr: this seems to be a little hack, as we do not
                             //    create a new node here with ->addItem but
                             //    simply pass the same node's address,
        }                    //    nevertheless this should be ok, as we add
                             //    several entries to the same node

        if (!empty ($catnode->categories)) {
            $this->create_tree_menu_mycourses($catnode->categories, $node);//Cr: recursively create nested nodes
        }

        if (!empty ($catnode->courses)) {
            foreach ($catnode->courses as $course) {

                $linkcss = '';//Cr: possible use: $linkcss = $course->visible ? '' : ' class="dimmed" ';

                if ($this->mycmyc) {//Cr: if admin or teacher wants to see his
                                    //    mycourses
                    if ($course->visible) {
                        $icon = $cmycicon;
                    } else {
                        $icon = $cinacticon;
                    }
                } else {
                    if (!$this->isadmin && !$this->isteacher) {
                        $icon = $cicon;
                    } else {
                        if ($course->visible) {
                            $icon = $cacticon;
                        } else {
                            $linkcss .= '';//Cr: possible use: $linkcss = $course->visible ? '' : ' class="dimmed" ';
                            $icon = $cinacticon;
                        }
                    }
                }

				if ($CFG->block_sitenavigation_mycourses_showlongnames) {//Cr: show long name and short in tool tip text
					$url = $CFG->wwwroot . '/course/view.php?id=' . $course->id .
				   '" title="' . htmlspecialchars(format_string($course->shortname), ENT_QUOTES) . $linkcss;
				   $text = format_string($course->fullname);
				} else {//Cr: show short name and long in tool tip text
					$url = $CFG->wwwroot . '/course/view.php?id=' . $course->id .
				   '" title="' . htmlspecialchars(format_string($course->fullname), ENT_QUOTES) . $linkcss;
				   $text = format_string($course->shortname);
				}

                if ($this->course->id == $course->id) {//Cr: if this is the actually displayed course page
                    $url = '';//Cr: show no link and no tool tip
                    $cssclass = 'treeMenuBold';
                } else {
                    $cssclass = 'treeMenuDefault';
                }

//Cr: linkTarget added to display Moodle pages always in a same/new window/tab
				$node->addItem(new HTML_TreeNode(array (
					'text' => $text,
					'link' => $url,
					'icon' => $icon,
				    'linkTarget' => 'mycourses',
					'cssClass' => $cssclass
				)));
            }
        }
    }
}


/** PURELY EXPERIMENTAL until thoroughly tested. The base is from course_list.
 *
 * Cr: added to mycourses code to better complete the mycourses list
 *
 * @uses $CFG
 * @uses $USER
 *
 * @var array $remotecourses to hold eventual mnet courses
 * @return array $remotecourses
 */

function get_remote_courses() {
    global $CFG, $USER;

$remotecourses = array();//Cr: initialize to empty array

// only for logged in users!
    if (!is_enabled_auth('mnet') || !isloggedin() || isguest() ) {
// no need to query anything remote related
        return $remotecourses;//Cr: only tested with get_records('course'),
    }                         //    which works fine here.

    if ($remotecourses = get_my_remotecourses()) {
// if we got courses, we are done
        return $remotecourses;
    }

//Cr: Do not understand why we would list mNet hosts if we do not have any
//    courses on them. This could be convenient though to see we have mNet
//    hosts but no courses on them.
    if ($hosts = get_my_remotehosts()) {
        $remotecourses[] = get_string('remotemoodles','mnet');
        foreach($USER->mnet_foreign_host_array as $somehost) {
            $remotecourses[] = $somehost['count'].get_string('courseson','mnet').' '.$somehost['url'];
        }
    }
    return $remotecourses;
}


/**
 * Convenience function - lists courses that a user has access to view.
 *
 * NOTE: this is a copy of the original get_my_courses function in
 * /lib/datalib.php with a capability passed as parameter whereas
 * the original checks statically for /moodle/course:view capability.
 * We need it to check for admins own courses, as proposed in the original:
 *
 * For admins and others with access to "every" course in the system, we should
 * try to get courses with explicit RAs.
 *
 * NOTE: this function is heavily geared towards the perspective of the user
 *       passed in $userid. So it will hide courses that the user cannot see
 *       (for any reason) even if called from cron or from another $USER's
 *       perspective.
 *
 *       If you really want to know what courses are assigned to the user,
 *       without any hiding or scheming, call the lower-level
 *       get_user_courses_bycap().
 *
 *
 * Notes inherited from get_user_courses_bycap():
 *
 * - $fields is an array of fieldnames to ADD
 *   so name the fields you really need, which will
 *   be added and uniq'd
 *
 * - the course records have $c->context which is a fully
 *   valid context object. Saves you a query per course!
 *
 * @uses $CFG,$USER
 * @param int $userid The user of interest
 * @param string $cap the capability to check for
 * @param string $sort the sortorder in the course table
 * @param array $fields - names of _additional_ fields to return (also accepts a string)
 * @param bool $doanything True if using the doanything flag
 * @param int $limit Maximum number of records to return, or 0 for unlimited
 * @return array {@link $COURSE} of course objects
 */
function get_my_courses_bycap($userid, $cap='moodle/course:view', $sort='visible DESC,sortorder ASC', $fields=NULL, $doanything=false,$limit=0) {

    global $CFG,$USER;

    // Guest's do not have any courses
    $sitecontext = get_context_instance(CONTEXT_SYSTEM);
    if (has_capability('moodle/legacy:guest',$sitecontext,$userid,false)) {
        return(array());
    }

    $basefields = array('id', 'category', 'sortorder',
                        'shortname', 'fullname', 'idnumber',
                        'teacher', 'teachers', 'student', 'students',
                        'guest', 'startdate', 'visible',
                        'newsitems',  'cost', 'enrol',
                        'groupmode', 'groupmodeforce');

    if (!is_null($fields) && is_string($fields)) {
        if (empty($fields)) {
            $fields = $basefields;
        } else {
            // turn the fields from a string to an array that
            // get_user_courses_bycap() will like...
            $fields = explode(',',$fields);
            $fields = array_map('trim', $fields);
            $fields = array_unique(array_merge($basefields, $fields));
        }
    } elseif (is_array($fields)) {
        $fields = array_unique(array_merge($basefields, $fields));
    } else {
        $fields = $basefields;
    }

    $orderby = '';
    $sort    = trim($sort);
    if (!empty($sort)) {
        $rawsorts = explode(',', $sort);
        $sorts = array();
        foreach ($rawsorts as $rawsort) {
            $rawsort = trim($rawsort);
            if (strpos($rawsort, 'c.') === 0) {
                $rawsort = substr($rawsort, 2);
            }
            $sorts[] = trim($rawsort);
        }
        $sort = 'c.'.implode(',c.', $sorts);
        $orderby = "ORDER BY $sort";
    }

//Cr: commented out this section, because the few cached courses would yield
//    ALL courses for admins due to their VIEW capability on every course

/*
    //
    // Logged-in user - Check cached courses
    //
    // NOTE! it's a _string_ because
    // - it's all we'll ever use
    // - it serialises much more compact than an array
    //   this a big concern here - cost of serialise
    //   and unserialise gets huge as the session grows
    //
    // If the courses are too many - it won't be set
    // for large numbers of courses, caching in the session
    // has marginal benefits (costs too much, not
    // worthwhile...) and we may hit SQL parser limits
    // because we use IN()
    //
    if ($userid === $USER->id) {
        if (isset($USER->loginascontext)
            && $USER->loginascontext->contextlevel == CONTEXT_COURSE) {
            // list _only_ this course
            // anything else is asking for trouble...
            $courseids = $USER->loginascontext->instanceid;
        } elseif (isset($USER->mycourses)
                  && is_string($USER->mycourses)) {
            if ($USER->mycourses === '') {
                // empty str means: user has no courses
                // ... so do the easy thing...
                return array();
            } else {
                $courseids = $USER->mycourses;
            }
        }
        if (isset($courseids)) {
            // The data massaging here MUST be kept in sync with
            // get_user_courses_bycap() so we return
            // the same...
            // (but here we don't need to check has_cap)
            $coursefields = 'c.' .join(',c.', $fields);
            $sql = "SELECT $coursefields,
                           ctx.id AS ctxid, ctx.path AS ctxpath,
                           ctx.depth as ctxdepth, ctx.contextlevel AS ctxlevel,
                           cc.path AS categorypath
                    FROM {$CFG->prefix}course c
                    JOIN {$CFG->prefix}course_categories cc
                      ON c.category=cc.id
                    JOIN {$CFG->prefix}context ctx
                      ON (c.id=ctx.instanceid AND ctx.contextlevel=".CONTEXT_COURSE.")
                    WHERE c.id IN ($courseids)
                    $orderby";
            $rs = get_recordset_sql($sql);
            $courses = array();
            $cc = 0; // keep count
            while ($c = rs_fetch_next_record($rs)) {
                // build the context obj
                $c = make_context_subobj($c);

                if ($limit > 0 && $cc >= $limit) {
                    break;
                }

                $courses[$c->id] = $c;
                $cc++;
            }
            rs_close($rs);
            return $courses;
        }
    }
*/
    // Non-cached - get accessinfo
    if ($userid === $USER->id && isset($USER->access)) {
//Cr: for debugging. Unfortunately the cached mycourses for admins are ALL courses because of VIEW capability
//print_r(array('get non cached courses'));
        $accessinfo = $USER->access;
    } else {
        $accessinfo = get_user_access_sitewide($userid);
    }


    $courses = get_user_courses_bycap($userid, $cap, $accessinfo,
                                      $doanything, $sort, $fields,
                                      $limit);

    $cats = NULL;
    // If we have to walk category visibility
    // to eval course visibility, get the categories
    if (empty($CFG->allowvisiblecoursesinhiddencategories)) {
        $sql = "SELECT cc.id, cc.path, cc.visible,
                       ctx.id AS ctxid, ctx.path AS ctxpath,
                       ctx.depth as ctxdepth, ctx.contextlevel AS ctxlevel
                 FROM {$CFG->prefix}course_categories cc
                 JOIN {$CFG->prefix}context ctx ON (cc.id = ctx.instanceid)
                WHERE ctx.contextlevel = ".CONTEXT_COURSECAT."
             ORDER BY cc.id";
        $rs = get_recordset_sql($sql);

        // Using a temporary array instead of $cats here, to avoid a "true" result when isnull($cats) further down
        $categories = array();
        while ($course_cat = rs_fetch_next_record($rs)) {
            // build the context obj
            $course_cat = make_context_subobj($course_cat);
            $categories[$course_cat->id] = $course_cat;
        }
        rs_close($rs);

        if (!empty($categories)) {
            $cats = $categories;
        }

        unset($course_cat);
    }
    //
    // Strangely, get_my_courses() is expected to return the
    // array keyed on id, which messes up the sorting
    // So do that, and also cache the ids in the session if appropriate
    //
    $kcourses = array();
    $courses_count = count($courses);
    $cacheids = NULL;
    $vcatpaths = array();
    if ($userid === $USER->id && $courses_count < 500) {
        $cacheids = array();
    }
    for ($n=0; $n<$courses_count; $n++) {

        //
        // Check whether $USER (not $userid) can _actually_ see them
        // Easy if $CFG->allowvisiblecoursesinhiddencategories
        // is set, and we don't have to care about categories.
        // Lots of work otherwise... (all in mem though!)
        //
        $cansee = false;
        if (is_null($cats)) { // easy rules!
            if ($courses[$n]->visible == true) {
                $cansee = true;
            } elseif (has_capability('moodle/course:viewhiddencourses',
                                     $courses[$n]->context, $USER->id)) {
                $cansee = true;
            }
        } else {
            //
            // Is the cat visible?
            // we have to assume it _is_ visible
            // so we can shortcut when we find a hidden one
            //
            $viscat = true;
            $cpath = $courses[$n]->categorypath;
            if (isset($vcatpaths[$cpath])) {
                $viscat = $vcatpaths[$cpath];
            } else {
                $cpath = substr($cpath,1); // kill leading slash
                $cpath = explode('/',$cpath);
                $ccct  = count($cpath);
                for ($m=0;$m<$ccct;$m++) {
                    $ccid = $cpath[$m];
                    if ($cats[$ccid]->visible==false) {
                        $viscat = false;
                        break;
                    }
                }
                $vcatpaths[$courses[$n]->categorypath] = $viscat;
            }

            //
            // Perhaps it's actually visible to $USER
            // check moodle/category:viewhiddencategories
            //
            // The name isn't obvious, but the description says
            // "See hidden categories" so the user shall see...
            // But also check if the allowvisiblecoursesinhiddencategories setting is true, and check for course visibility
            if ($viscat === false) {
                $catctx = $cats[$courses[$n]->category]->context;
                if (has_capability('moodle/category:viewhiddencategories', $catctx, $USER->id)) {
                    $vcatpaths[$courses[$n]->categorypath] = true;
                    $viscat = true;
                } elseif ($CFG->allowvisiblecoursesinhiddencategories && $courses[$n]->visible == true) {
                    $viscat = true;
                }
            }

            //
            // Decision matrix
            //
            if ($viscat === true) {
                if ($courses[$n]->visible == true) {
                    $cansee = true;
                } elseif (has_capability('moodle/course:viewhiddencourses',
                                        $courses[$n]->context, $USER->id)) {
                    $cansee = true;
                }
            }
        }
        if ($cansee === true) {
            $kcourses[$courses[$n]->id] = $courses[$n];
            if (is_array($cacheids)) {
                $cacheids[] = $courses[$n]->id;
            }
        }
    }
    if (is_array($cacheids)) {
        // Only happens
        // - for the logged in user
        // - below the threshold (500)
        // empty string is _valid_
        $USER->mycourses = join(',',$cacheids);
    } elseif ($userid === $USER->id && isset($USER->mycourses)) {
        // cheap sanity check
        unset($USER->mycourses);
    }

    return $kcourses;
}

}

?>
